@@ -1,7 +1,7 @@
 ##############################################################################
 ##
 ##  Web::Reactor application machinery
-##  2013 (c) Vladi Belperchinov-Shabanski "Cade"
+##  2013-2016 (c) Vladi Belperchinov-Shabanski "Cade"
 ##  <cade@bis.bg> <cade@biscom.net> <cade@cpan.org>
 ##
 ##  LICENSE: GPLv2
@@ -10,6 +10,8 @@
 package Web::Reactor::Actions;
 use strict;
 
+use parent 'Web::Reactor::Base'; 
+
 sub new
 {
   my $class = shift;
@@ -36,6 +38,13 @@ sub new
 #       result text to be replaced in output
 sub call { die "Web::Reactor::Acts::*::call() is not implemented!"; }
 
+#sub DESTROY
+#{
+# my $self = shift;
+#
+# print "DESTROY: Reactor: $self\n";
+#}
+
 ##############################################################################
 1;
 ###EOF########################################################################
